"""
Provides a WSGI interface for serving web requests. Uses Flask
"""

from flask import request, send_file, g
from io import BytesIO
from werkzeug.datastructures import CombinedMultiDict, MultiDict

class http_server:
	"""
	Designed to be used in the function decorated by wsgi_interface
	"""

	def __init__( self, content_type = 'application/json' ):
		self.content_type = content_type
		# create a default response, in case put isn't called
		g.simpletransfers_wsgi_response = ''

	def get( self ):
		files = dict(
			[ ( f.name, f.read() ) for f in request.files.values() ]
		)
		return CombinedMultiDict( [
			request.values,
			MultiDict( g.simpletransfers_wsgi_request ),
			MultiDict( files ),
			MultiDict( { '_raw': request.data } ),
		] )

	def put( self, data, name ):
		g.simpletransfers_wsgi_response = send_file(
			BytesIO( data ),
			self.content_type,
			attachment_filename = name,
		)

def wsgi_interface( path_function ):
	"""
	Usage:
	@app.route( '/path/to/api' )
	@wsgi_interface
	def path_function( ... )
	"""
	def wrapper( *args, **kwargs ):
		g.simpletransfers_wsgi_request = kwargs
		path_function( *args )
		return g.simpletransfers_wsgi_response
	return wrapper



import unittest

from testftp import testFTP
from testsftp import testSFTP
from testhttpclient import testHTTPClient
from testhttpserver import testHTTPServer
from testlocal import testLocal
from testmail import testMail
from testsoap import testSOAP
from testzip import testZipfile

def suite():
	test_suite = unittest.TestSuite()

	test_suite.addTest( unittest.makeSuite(testFTP) )
	test_suite.addTest( unittest.makeSuite(testSFTP) )
	test_suite.addTest( unittest.makeSuite(testHTTPClient) )
	test_suite.addTest( unittest.makeSuite(testHTTPServer) )
	test_suite.addTest( unittest.makeSuite(testLocal) )
	test_suite.addTest( unittest.makeSuite(testMail) )
	test_suite.addTest( unittest.makeSuite(testSOAP) )
	test_suite.addTest( unittest.makeSuite(testZipfile) )

	return test_suite

if __name__ == '__main__':
	runner = unittest.TextTestRunner()
	runner.run( suite() )



import unittest
from mock import Mock, patch, ANY

from simpletransfers.sftp import sftp, TransferException

class testSFTP( unittest.TestCase ):

	test_data = 'eonfeoiwnfioenmfoieapow kjpojfwpfwdfls;DL'

	def setUp(self):
		trans_patcher = patch( 'paramiko.Transport', autospec = True )
		conn_patcher = patch( 'paramiko.SFTPClient', autospec = True )

		self.mock_transport_constructor = trans_patcher.start()
		self.mock_transport = self.mock_transport_constructor.return_value
		self.mock_sftp = conn_patcher.start().from_transport( self.mock_transport )

		self.mock_sftp.listdir = Mock(
			return_value = [ 'test.xml', 'test.txt' ]
		)
		self.mock_file = Mock()
		self.mock_file.read = Mock( return_value = self.test_data )
		self.mock_sftp.file = Mock( return_value = self.mock_file )

		self.addCleanup( conn_patcher.stop )
		self.addCleanup( trans_patcher.stop )

	def test_get( self ):
		obj = sftp( 'local.test', 'user1', 'passwd' )
		contents = obj.get( r'.*\.xml' )

		self.mock_transport_constructor.assert_called_with( 'local.test:22' )
		self.mock_transport.connect.assert_called_with(
			username = 'user1', password = 'passwd'
		)
		self.mock_sftp.file.assert_called_with( './test.xml', 'rb' )

		self.assertIn( 'test.xml', contents )
		self.assertNotIn( 'test.txt', contents )
		self.assertEquals( self.test_data, contents[ 'test.xml' ] )

	def test_get_with_processed( self ):
		obj = sftp( 'local.test', None, None, processed = 'done' )
		contents = obj.get()

		self.mock_sftp.rename.assert_any_call( './test.xml', 'done/test.xml' )
		self.mock_sftp.rename.assert_any_call( './test.txt', 'done/test.txt' )

	def test_put( self ):
		obj = sftp( 'local.test', 'user1', 'passwd', permissions = 0777 )
		obj.put( self.test_data, 'test.txt' )

		self.mock_transport_constructor.assert_called_with( 'local.test:22' )
		self.mock_transport.connect.assert_called_with(
			username = 'user1', password = 'passwd'
		)

		self.mock_sftp.file.assert_called_with( './test.txt.uploading', 'wb' )
		self.mock_file.write.assert_called_with( self.test_data )

		self.mock_sftp.rename.assert_called_with(
			'./test.txt.uploading', './test.txt'
		)

	def test_put_no_rename( self ):
		obj = sftp( 'local.test', 'user1', 'passwd' )
		obj.put( self.test_data, 'test.txt', rename = False )

		self.mock_sftp.file.assert_called_with(
			'./test.txt', 'wb'
		)
		self.mock_file.write.assert_called_with( self.test_data )
		self.assertEqual( self.mock_sftp.rename.call_count, 0 )

	def test_connection_error( self ):
		self.mock_transport.connect = Mock(
			side_effect = Exception( 'wibble' )
		)

		obj = sftp( 'local.test', None, None )
		with self.assertRaises( TransferException ):
			obj.put( None, '' )

if __name__ == '__main__':
    unittest.main()



"""
"""

import contextlib
import ctxlogger
import logging
import suds
import re
import urllib2

from . import TransferException

logger = logging.getLogger( __name__ )
logger.addHandler( logging.NullHandler() )

class soap( object ):

	_wsdl = ''
	_client = None

	"""
	Make remote SOAP calls
	"""
	def __init__( self, wsdl, headers = None, timeout = 10 ):
		"""
		wsdl - the WDSL for this connection
		"""
		self._wsdl = wsdl
		self._client = suds.client.Client( self._wsdl, timeout = timeout )
		if headers:
			self._client.set_options( soapheaders = headers )

	def put( self, data, name ):
		"""
		data - the message to send to the remote method
		name - the name of the method to call
		"""
		with contextlib.nested(
			ctxlogger.context( 'service', str(self._wsdl) ),
			ctxlogger.context( 'function', name )
		):
			try:
				remote_func = getattr( self._client.service, name )
			except AttributeError as e:
				ctxlogger.exception(
					TransferException, 'Unrecognised Function', orig_exc = e
				)

			try:
				return remote_func( data )
			except suds.WebFault as e:
				ctxlogger.exception( TransferException, str(e), orig_exc = e )
			except urllib2.URLError as e:
				if 'timed out' in str(e):
					return
				ctxlogger.exception( TransferException, str(e), orig_exc = e )

	def get( self, name ):
		"""
		name - the remote method to call
		"""
		return self.put( None, name )



"""
"""

import ctxlogger
import logging
import os
import re
import requests

from . import TransferException

logger = logging.getLogger( __name__ )
logger.addHandler( logging.NullHandler() )

class http_client( object ):
	"""
	Handle HTTP client transfers
	"""

	def __init__( self,
		url,
		content_type = 'application/json',
		debug = False,
		headers = {},
	):

		self.address = url.lower()

		# content-type is such a common option it's given its own arg
		# so roll it in here
		self.headers = headers
		self.headers.update( { 'Content-Type': content_type } )

	def get( self, params = {} ):
		"""
		Fetch data from a URL via a GET request

		params - HTTP parameters to submit with the request (default: {})
		"""
		with ctxlogger.context( 'address', self.address ):
			return self._request( requests.get, params )

	def put( self, data, name = None, params = {} ):
		"""
		Put data to a URL via a POST request

		data - the data to send
		name - the filename of the data (currently ignored)
		params - HTTP parameters to submit with the request
		"""
		with ctxlogger.context( 'address', self.address ):
			return self._request( requests.post, params, data )

	def _request( self, method, params = {}, data = None ):
		"""
		Send a request and handle the response
		"""
		try:
			response = method(
				self.address,
				params = params,
				headers = self.headers,
				data = data,
			)
		except requests.exceptions.Timeout:
			return
		except requests.exceptions.RequestException as e:
			ctxlogger.exception( TransferException, str(e), orig_exc = e )

		if response.status_code != requests.codes.ok:
			ctxlogger.exception(
				TransferException, 
				'{} {}'.format( response.status_code, response.text ),
			)

		filename = os.path.basename( self.address )

		# was a filename specified?
		try:
			disposition = response.headers[ 'Content-Disposition' ]
		except KeyError:
			pass
		else:
			match = re.search( 'filename="([^"])"', disposition )
			if match:
				filename = match.group(1)

		return { filename: response.content }



import os
import subprocess
import unittest

from simpletransfers import local, TransferException

testfiles = { 'one.txt': 'wibble\n' }

class testLocal( unittest.TestCase ):

	def setUp( self ):
		self.local_dir = os.path.dirname( __file__ )

	def tearDown( self ):
		try:
			os.unlink( os.path.join( self.local_dir, 'local.test' ) )
		except:
			pass
		try:
			os.unlink( os.path.join( self.local_dir, 'newdir', 'local.test' ) )
		except:
			pass
		try:
			os.rmdir( os.path.join( self.local_dir, 'newdir' ) )
		except:
			pass

	def test_get( self ):
		files = local(
			source = self.local_dir, debug = True
		).get(
			r'.*\.txt'
		)
		self.assertEquals( files, testfiles )

	def test_get_nonexistant_directory( self ):
		with self.assertRaises( TransferException ):
			files = local(
				source = '/nope', debug = True
			).get(
				r'.*\.txt'
			)

	def test_get_file_not_directory( self ):
		with self.assertRaises( TransferException ):
			files = local(
				source = __file__, debug = True
			).get(
				r'.*\.txt'
			)

	def test_get_with_processed( self ):
		with open( os.path.join( self.local_dir, 'local.test' ), 'w' ) as outf:
			outf.write( '' )

		files = local(
			source = self.local_dir, processed = 'processed',
		).get(
			r'.*\.test$'
		)
		self.assertTrue(
			os.path.isfile(
				os.path.join( self.local_dir, 'processed', 'local.test' )
			)
		)
		os.unlink( os.path.join( self.local_dir, 'processed', 'local.test' ) )
		os.rmdir( os.path.join( self.local_dir, 'processed' ) )

	def test_get_with_delete( self ):
		with open( os.path.join( self.local_dir, 'local.test' ), 'w' ) as outf:
			outf.write( '' )

		files = local(
			source = self.local_dir,
		).get(
			r'.*\.test$'
		)
		self.assertFalse(
			os.path.exists(
				os.path.join( self.local_dir, 'local.test' )
			)
		)

	def test_put( self ):
		local(
			destination = self.local_dir, debug = True
		).put(
			'UNITTESTING!', 'local.test'
		)
		with open( os.path.join( self.local_dir, 'local.test' ) ) as fh:
			self.assertEquals( fh.read(), 'UNITTESTING!' )

	def test_put_nonexistant_directory( self ):
		dstdir = os.path.join( self.local_dir, 'newdir' )
		local(
			destination = dstdir, debug = True
		).put(
			'UNITTESTING!', 'local.test'
		)
		with open( os.path.join( dstdir, 'local.test' ) ) as fh:
			self.assertEquals( fh.read(), 'UNITTESTING!' )

	def test_put_nopermissions( self ):
		with self.assertRaises( TransferException ):
			local(
				destination = '/nonexistant', debug = True,
			).put(
				'UNITTESTING!', 'local.test'
			)

		with self.assertRaises( TransferException ):
			local(
				destination = '/root', debug = True,
			).put(
				'UNITTESTING!', 'local.test'
			)

if __name__ == '__main__':
	unittest.main()



"""
A set of classes that provide operations to
get files or put files to or from somewhere,
usually a network resource.

All these classes implement a get and put method,
for, well, getting and putting data.
"""
import os
import sys

class TransferException( Exception ):
	pass

from .local import local
from .ftp import ftp
from .sftp import sftp
from .mail import mail
from .http_client import http_client
from .http_server import http_server
from .zip import zipfile
from .soap import soap



"""
"""

import contextlib
import ctxlogger
import datetime
import logging
import os
import paramiko
import re

from . import TransferException

logger = logging.getLogger( __name__ )
logger.addHandler( logging.NullHandler() )

class sftp( object ):

	_conn = None
	_user = ''
	_password = ''
	_host = ''
	_port = 22
	_source = '.'
	_destination = '.'
	_processed = ''
	_debug = False
	_permissions = 0o777

	"""
	Secure (via SSH) FTP
	"""
	def __init__( self,
		host,
		user,
		password,
		port = 22,
		source = '.',
		destination = '.',
		processed = None,
		debug = False,
		permissions = None,
	):
		"""
		host - the server name
		user - the SFTP username
		password - the SFTP password
		port - the server port (default 22)
		source - the remote directory to look for files (default '.')
		destination - the remote directory to put files (default '.')
		processed - the remote directory to move fetched files to (optional)
		debug - leave remote files after fetching (default False)
		permissions - chmod mask for uploaded files (default None)
		"""
		self._address = 'sftp://{}:{}/{}'.format(
			self._host, self._port, self._destination
		)

		self._user = user
		self._password = password
		self._host = host
		self._port = port

		self._source = source
		self._destination = destination
		self._processed = processed

		self._debug = debug
		self._permissions = permissions

	def put( self, data, name, rename = True ):
		# automatically try to format current timestamp in name
		# if no date format strings, will return original
		name = datetime.datetime.now().strftime( name )

		with ctxlogger.context( 'address', self._address ):
			self.connect()

			with ctxlogger.context( 'file', name ):

				outfn = os.path.join( self._destination, name )

				logger.info( 'Putting' )

				if not rename:
					try:
						rfh = self._conn.file( outfn, 'wb' )
						rfh.write( data )
						rfh.close()
					except Exception as e:
						ctxlogger.exception(
							TransferException,
							"Could not write data: {}".format( str(e) ),
							orig_exc = e,
						)
				else:
					tmpfn = '{}.uploading'.format( outfn )

					# write date to a temporary filename first
					# to minimise risk of partially sent file
					# being picked up
					try:
						rfh = self._conn.file( tmpfn, 'wb' )
						rfh.write( data )
						rfh.close()
					except Exception as e:
						ctxlogger.exception(
							TransferException,
							"Could not write data: {}".format( str(e) ),
							orig_exc = e
						)

					# then rename once we're done (an atomic op)
					logger.debug( 'Renaming temporary file' )
					try:
						self._conn.rename( tmpfn, outfn )
					except Exception as e:
						ctxlogger.exception(
							TransferException,
							"Could not rename file: {}".format( str(e) ),
							orig_exc = e
						)

				if self._permissions:
					try:
						self._conn.chmod( outfn, self._permissions )
					except Exception as e:
						ctxlogger.exception(
							TransferException,
							"Could not set permissions to {}: {}".format(
								self._permissions, str(e),
							),
							orig_exc = e
						)

	def get( self, pattern = '.*' ):
		"""
		pattern - a regexp to pick remote files (default '.*')
		"""
		data = {}

		with ctxlogger.context( 'address', self._address ):
			self.connect()
			pattern = re.compile( pattern )

			for remotef in self._conn.listdir( self._source ):
				with ctxlogger.context( 'file', remotef ):

					if not pattern.match( remotef ):
						logger.debug( 'Ignoring' )
						continue

					logger.info( 'Getting' )

					remotep = os.path.join( self._source, remotef )
					try:
						rfh = self._conn.file( remotep, 'rb' )
						data[ remotef ] = rfh.read()
						rfh.close()
					except Exception as e:
						ctxlogger.exception(
							TransferException,
							"Could not read data: {}".format( str(e) ),
							orig_exc = e
						)

					if not self._debug:
						if self._processed:
							logger.debug(
								'Moving to {}'.format( self._processed )
							)

							destp = os.path.join( self._processed, remotef )
							try:
								self._conn.rename( remotep, destp )
							except Exception as e:
								ctxlogger.exception(
									TransferException,
									'Could not move: {}'.format( str(e) ),
									orig_exc = e
								)
						else:
							logger.debug( 'Deleting' )
							try:
								self._conn.remove( remotep )
							except Exception as e:
								ctxlogger.exception(
									TransferException,
									"Could not delete: {}".format( str(e) ),
									orig_exc = e
								)

		return data

	def connect( self ):
		if self._conn:
			return

		try:
			transport = paramiko.Transport(
				'{}:{}'.format( self._host, self._port )
			)
			transport.connect(
				username = self._user, password = self._password
			)
			self._conn = paramiko.SFTPClient.from_transport( transport )
		except Exception as e:
			ctxlogger.exception(
				TransferException,
				"Could not connect: {}".format( str(e) ),
				orig_exc = e
			)



import localmail
import os
import sys
import threading

import unittest

from simpletransfers import mail, TransferException

thread = None

def setUpModule():
	global thread
	thread = threading.Thread(
		target = localmail.run,
		args = ( 2142, 2143, ),
	)
	thread.start()

def tearDownModule():
	localmail.shutdown_thread( thread )


testfiles = { 'one.txt': 'wibble\n' }

class testMail( unittest.TestCase ):

	def test_getput( self ):
		smtp = mail(
			'a', 'b', # auth ignored for test
			'localhost',
			2142,
			'noreply@casual-tempest.net',
			'acustomer@casual-tempest.net',
		)
		imap = mail(
			'a', 'b',
			'localhost',
			2143,
			'acustomer@casual-tempest.net',
			processed = 'processed',
		)

		smtp.put( 'wibble', 'test.txt' )

		msgs = imap.get( r'.*\.txt' )
		self.assertEqual( len(msgs), 1 )
		self.assertEqual( msgs.keys(), [ 'test.txt', ] )
		self.assertEqual( msgs.values(), [ 'wibble', ] )

	def test_put_no_destination( self ):
		smtp = mail(
			'a', 'b', # auth ignored for test
			'localhost',
			2142,
			'noreply@casual-tempest.net',
		)
		with self.assertRaises( TransferException ):
			smtp.put( 'wibble', 'test.txt' )

	def test_put_incorrect_connection( self ):
		smtp = mail(
			'a', 'b', # auth ignored for test
			'localhost',
			666,
			'noreply@casual-tempest.net',
			'noreply@casual-tempest.net',
		)
		with self.assertRaises( TransferException ):
			smtp.put( 'wibble', 'test.txt' )

	def test_get_incorrect_connection( self ):
		imap = mail(
			'a', 'b', # auth ignored for test
			'localhost',
			666,
			'noreply@casual-tempest.net',
		)
		with self.assertRaises( TransferException ):
			imap.get()

if __name__ == '__main__':
	unittest.main()



"""
"""

import contextlib
import ctxlogger
import datetime
import os
import logging
import re
import ssl
import sys
import ftplib

from io import BytesIO

from . import TransferException

logger = logging.getLogger( __name__ )
logger.addHandler( logging.NullHandler() )

class ftp( object ):

	_conn = None
	_host = ''
	_port = 21
	_user = ''
	_password = ''
	_source = '.'
	_destination = '.'
	_processed = ''
	_timeout = 30
	_ftps = False
	_debug = False
	_file_contents = ''

	"""
	Basic (non-secure) FTP transport
	"""
	def __init__( self,
		host,
		user,
		password,
		port = 21,
		source = '.',
		destination = '.',
		processed = None,
		debug = False,
		timeout = 30,
		ftps = False,
	):
		"""
		host - the server name
		port - the server port (default 21)
		user - the FTP username
		password - the FTP password
		source - the remote directory to get files (default '.')
		destination - the remote directory to put files (default '.')
		processed - the remote directory to move files to
			after being fetched (optional)
		debug - leave remote files when fetched (default False)
		timeout - timeout wait period in seconds (default 30)
		"""
		self._address = 'ftp://{}:{}/{}'.format( host, port, destination )

		self._host = host
		self._port = port
		self._timeout = timeout
		self._user = user
		self._password = password

		self._source = source
		self._destination = destination
		self._processed = processed

		self._ftps = ftps

		self._debug = debug

	def _connect( self ):
		if self._conn != None:
			return

		if not self._ftps:
			self._conn = ftplib.FTP()
		else:
			self._conn = ftplib.FTP_TLS()
			self._conn.ssl_version = ssl.PROTOCOL_SSLv23

		with ctxlogger.context( 'address', self._address ):
			logger.info( 'Connecting' )
			try:
				self._conn.connect( self._host, self._port, self._timeout )
				self._conn.login( self._user, self._password )
			except Exception as e:
				ctxlogger.exception( TransferException, str(e), orig_exc = e )

		if self._ftps:
			self._conn.prot_p()

	def put( self, data, name, rename = True ):
		# automatically try to format current timestamp in
		# if no date format strings, will return original
		name = datetime.datetime.now().strftime( name )

		self._connect()

		with contextlib.nested(
			ctxlogger.context( 'address', self._address ),
			ctxlogger.context( 'file', name )
		):

			outfn = os.path.join( self._destination, name )

			logger.info( 'Putting' )
			if rename:
				tmpfn = '{}.uploading'.format( outfn )

				try:
					# write file to temporary name to minimise chance
					# of being picked up mid-upload
					self._conn.storbinary(
						'STOR {}'.format( tmpfn ),
						BytesIO( data )
					)

					# now rename file (an atmoic op) once we're done
					logger.debug( 'Renaming' )
					self._conn.rename( tmpfn, outfn )
				except Exception as e:
					ctxlogger.exception( TransferException, str(e), orig_exc = e )
			else:
				try:
					self._conn.storbinary(
						'STOR {}'.format( outfn ),
						BytesIO( data )
					)
				except Exception as e:
					ctxlogger.exception( TransferException, str(e) )

	def get( self, pattern = '.*' ):
		"""
		pattern - a regexp to pick remote files (default '.*')
		"""
		data = {}

		self._connect()

		with ctxlogger.context( 'address', self._address ):

			pattern = re.compile( pattern )

			for f in self._conn.nlst( self._source ):
				with ctxlogger.context( 'file', f ):

					remotef = os.path.basename( f.replace( '\\', '/' ) )
					if not pattern.match( remotef ):
						logger.debug( 'Ignoring, does not match pattern' )
						continue

					logger.info( 'Getting' )

					self._file_contents = ''
					try:
						self._conn.retrbinary(
							'RETR {}'.format( f ), self._download
						)
					except Exception as e:
						ctxlogger.exception(
							TransferException, str(e), orig_exc = e
						)

					data[ remotef ] = self._file_contents

					if not self._debug:
						if self._processed:
							logger.info(
								'Moving to {}'.format( self._processed )
							)

							destp = os.path.join( self._processed, remotef )
							try:
								self._conn.rename( f, destp )
							except Exception as e:
								ctxlogger.exception(
									TransferException, str(e), orig_exc = e
								)
						else:
							logger.info( 'Deleting' )
							try:
								self._conn.delete( f )
							except Exception as e:
								ctxlogger.exception(
									TransferException, str(e), orig_exc = e
								)

		return data

	def _download( self, data ):
		self._file_contents += data



"""
"""

import contextlib
import ctxlogger
import datetime
import email
import imaplib
import logging
import mimetypes
import re
import smtplib

from email import encoders
from email.mime.base import MIMEBase
from email.mime.image import MIMEImage
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText

from . import TransferException

logger = logging.getLogger( __name__ )
logger.addHandler( logging.NullHandler() )

class mail( object ):
	"""
	Sends or receives data via email attachments
	"""

	def __init__( self,
		user,
		password,
		host,
		port = '143',
		source = None,
		destination = None,
		processed = None,
		debug = False,
	):
		"""
		user - the email account user name
		password - the email account password
		host - the email server
		port - the email server port (default 143)
		source - the email address to send files from
		destination - the email address(es) to send files to
		processed - the email folder to move fetched mails to (optional)
		debug - leave emails after fetching (default False)
		"""
		self._user = user.encode( u'ascii' ) # smtp lib is pedantic about enc
		self._password = password.encode( u'ascii' )
		self._host = host.encode( u'ascii' )
		self._port = str(port).encode( u'ascii' )

		self._source = source

		if destination and not isinstance( destination, list ):
			destination = [ destination ]
		self._destination = destination
		self._processed = processed

		self._debug = debug

		self._data = {}

	def put( self, data, name, subject = None ):
		# automatically try to format current timestamp in name
		# if no date format strings, will return original
		name = datetime.datetime.now().strftime( name )

		if not self._destination:
			ctxlogger.exception(
				TransferException, u'No destination addresses given'
			)

		if not subject:
			subject = name

		with contextlib.nested(
			ctxlogger.context( u'address', ', '.join( self._destination ) ),
			ctxlogger.context( u'file', name )
		):

			try:
				if self._port == 465: # with SSL
					conn = smtplib.SMTP_SSL( self._host, self._port )
				else: # without SSL
					conn = smtplib.SMTP( self._host, self._port )
				conn.login( self._user, self._password )
			except Exception as e:
				ctxlogger.exception( TransferException, str(e), orig_exc = e )

			email = MIMEMultipart()
			email[ u'Subject' ] = subject
			email[ u'From' ] = self._source
			email[ u'To' ] = ', '.join( self._destination )

			ctype, enc = mimetypes.guess_type( name )
			if ctype is None or enc is not None: # probably a compressed file
				ctype = u'application/octet-stream'
			maintype, subtype = ctype.split( '/', 1 )
			if maintype == u'text':
				msg = MIMEText( data, _subtype=subtype )
			elif maintype == u'image':
				msg = MIMEImage( data, _subtype=subtype )
			else:
				msg = MIMEBase( maintype, subtype )
				msg.set_payload( data )
				encoders.encode_base64( msg )

			msg.add_header(
				u'Content-Disposition', u'attachment',
				filename = name,
			)
			email.attach( msg )

			try:
				conn.sendmail(
					self._source, self._destination, email.as_string()
				)
			except Exception as e:
				ctxlogger.exception( TransferException, str(e), orig_exc = e )

	def _imap( self, action, *args, **kwargs ):
		"""
		Perfoms an IMAP action, and catches any errors.

		action - the action to perform
		args, kwargs - custom data for the IMAP action
		"""
		logger.debug(
			u'{}: {} / {}'.format( action, str(args), str(kwargs) )
		)
		result, data = getattr( self.conn, action )( *args, **kwargs  )
		if result != u'OK':
			ctxlogger.exception( TransferException, data )
		return data

	def _attachments( self, msg, pattern ):
		"""
		Find attachments in the given message.
		"""
		for part in msg.walk():
			tmp = part.get( u'Content-Disposition', u'' ).strip()
			if tmp.startswith( u'attachment;' ):
				fn = part.get_filename()

				with ctxlogger.context( u'file', fn ):
					if not pattern.match( fn ):
						logger.info( u'Ignoring' )
						continue

					self._data[ fn ] = part.get_payload( decode=True )

	def get( self, pattern = '.*' ):
		"""
		pattern - the filename pattern to match attachments against (default .*)
		"""

		with ctxlogger.context( u'address', self._user ):

			pattern = re.compile( pattern, re.I )

			try:
				if self._port == u'143' or self._port == u'2143': # without SSL
					self.conn = imaplib.IMAP4( self._host, self._port )
				else: # with SSL
					self.conn = imaplib.IMAP4_SSL( self._host, self._port )
				self.conn.login( self._user, self._password )
			except Exception as e:
				ctxlogger.exception( TransferException, str(e), orig_exc = e )

			self._imap( u'select' )
			data = self._imap( u'search', None, u'(NOT DELETED)' )
			indices = data[0].split() if data[0] else []

			indices.reverse()
			for idx in indices:
				with ctxlogger.context( u'mail', idx ):

					try:
						data = self._imap( u'fetch', idx, u'(RFC822)' )
						msg = email.message_from_string( data[0][1] )
					except Exception as e:
							ctxlogger.exception(
								TransferException, str(e), orig_exc = e
							)
					else:
						if msg.is_multipart():
							self._attachments( msg, pattern )

					if self._debug:
						continue

					if self._processed:
						if not self.conn.list( self._processed ):
							self._imap( u'create', self._processed )
						self._imap( u'copy', idx, self._processed )

					self._imap( u'store', idx, u'+FLAGS', r'(\Deleted)' )

			self._imap( u'expunge' )
			self.conn.close()
			self.conn.logout()

			return self._data



import unittest
from mock import Mock, patch, ANY
from suds import WebFault
from urllib2 import URLError

from simpletransfers.soap import soap, TransferException

class testSOAP( unittest.TestCase ):

	test_data = 'eonfeoiwnfioenmfoieapow kjpojfwpfwdfls;DL'

	def setUp(self):
		patcher = patch( 'suds.client.Client', autospec = True )

		self.mock_soap_constructor = patcher.start()
		self.mock_soap = self.mock_soap_constructor.return_value
		self.mock_soap.service = Mock()
		self.mock_soap.service.RemoteFunctionName = Mock(
			return_value = self.test_data
		)

		self.addCleanup( patcher.stop )

	def test_get( self ):
		obj = soap( 'local.test' )
		contents = obj.get( 'RemoteFunctionName' )

		self.mock_soap_constructor.assert_called_with(
			'local.test', timeout = 10
		)
		self.mock_soap.service.RemoteFunctionName.assert_called_with( None )

		self.assertEquals( self.test_data, contents )

	def test_with_headers( self ):
		obj = soap( 'local.test', headers = self.test_data )
		self.mock_soap.set_options.assert_called_with(
			soapheaders = self.test_data
		)

	def test_put( self ):
		obj = soap( 'local.test' )
		contents = obj.put( self.test_data, 'RemoteFunctionName' )

		self.mock_soap.service.RemoteFunctionName.assert_called_with(
			self.test_data
		)
		self.assertEquals( self.test_data, contents )

	def test_unrecognised_method( self ):
		del self.mock_soap.service.RemoteFunctionName
		obj = soap( 'local.test' )

		with self.assertRaises( TransferException ):
			obj.get( 'RemoteFunctionName' )

	def test_method_error( self ):
		self.mock_soap.service.RemoteFunctionName.side_effect =\
			WebFault( None, None )
		obj = soap( 'local.test' )

		with self.assertRaises( TransferException ):
			obj.get( 'RemoteFunctionName' )

	def test_method_timeout( self ):
		self.mock_soap.service.RemoteFunctionName.side_effect =\
			URLError( 'timed out' )
		obj = soap( 'local.test' )

		self.assertEquals( obj.get( 'RemoteFunctionName' ), None )

if __name__ == '__main__':
    unittest.main()



import unittest
from mock import Mock, patch, ANY

from simpletransfers.ftp import ftp, TransferException

class testFTP( unittest.TestCase ):

	test_data = 'eonfeoiwnfioenmfoieapow kjpojfwpfwdfls;DL'

	def setUp(self):
		patcher = patch( 'ftplib.FTP', autospec = True )

		def mock_retrbinary( cmd, func ):
			func( self.test_data )

		def mock_storbinary( cmd, stringio ): # can't seem to mock StringIO :/
			self.ret_data = stringio.getvalue()

		self.mock_ftp = patcher.start().return_value
		self.mock_ftp.nlst = Mock( return_value = [ 'test.xml', 'test.txt' ] )
		self.mock_ftp.retrbinary = Mock( side_effect = mock_retrbinary )
		self.mock_ftp.storbinary = Mock( side_effect = mock_storbinary )

		self.addCleanup( patcher.stop )

		self.ret_data = ''

	def test_get( self ):
		obj = ftp( 'local.test', 'user1', 'passwd' )
		contents = obj.get( r'.*\.xml' )

		self.mock_ftp.connect.assert_called_with( 'local.test', 21, 30 )
		self.mock_ftp.login.assert_called_with( 'user1', 'passwd' )

		self.assertIn( 'test.xml', contents )
		self.assertNotIn( 'test.txt', contents )
		self.assertEquals( self.test_data, contents[ 'test.xml' ] )

	def test_get_with_processed( self ):
		obj = ftp( 'local.test', None, None, processed = 'done' )
		contents = obj.get()

		self.mock_ftp.rename.assert_any_call( 'test.xml', 'done/test.xml' )
		self.mock_ftp.rename.assert_any_call( 'test.txt', 'done/test.txt' )

	def test_put( self ):
		obj = ftp( 'local.test', 'user1', 'passwd' )
		obj.put( self.test_data, 'test.txt' )

		self.mock_ftp.connect.assert_called_with( 'local.test', 21, 30 )
		self.mock_ftp.login.assert_called_with( 'user1', 'passwd' )
		self.mock_ftp.storbinary.assert_called_with(
			'STOR ./test.txt.uploading', ANY
		)
		self.assertEqual( self.ret_data, self.test_data )
		self.mock_ftp.rename.assert_called_with(
			'./test.txt.uploading', './test.txt'
		)

	def test_put_no_rename( self ):
		obj = ftp( 'local.test', 'user1', 'passwd' )
		obj.put( self.test_data, 'test.txt', rename = False )

		self.mock_ftp.storbinary.assert_called_with(
			'STOR ./test.txt', ANY
		)
		self.assertEqual( self.ret_data, self.test_data )
		self.assertEqual( self.mock_ftp.rename.call_count, 0 )

	def test_connection_error( self ):
		self.mock_ftp.connect = Mock( side_effect = Exception( 'wibble' ) )

		obj = ftp( 'local.test', None, None )
		with self.assertRaises( TransferException ):
			obj.put( None, '' )

if __name__ == '__main__':
    unittest.main()



"""
"""

import contextlib
import ctxlogger
import datetime
#import inotify
import logging
import os
import re
import threading
import time

#from inotify import watcher

from . import TransferException

logger = logging.getLogger( __name__ )
logger.addHandler( logging.NullHandler() )

class local( object ):
	"""
	Input from or output to a local directory
	"""
	def __init__( self,
		source = '.',
		destination = '.',
		processed = None,
		perms = 0o600,
		debug = False
	):
		"""
		source - the directory to put or get files (default '.')
		destination - the directory to put files (default '.')
		processed - the directory to move collected files to (default None)
		perms - default permissions on created files (default '0600')
		debug - leave files as we process them? (default False)
		"""
		self._source = os.path.abspath( source )
		self._destination = os.path.abspath( destination )
		if processed != None and not os.path.isabs( processed ):
			self._processed = os.path.join( self._source, processed )
		else:
			self._processed = processed
		self._perms = perms
		self._debug = debug

	def put( self, data, name ):
		# automatically try to format current timestamp in name
		# if no date format strings, will return original
		name = datetime.datetime.now().strftime( name )

		with ctxlogger.context( 'address', self._destination ):
			if not os.path.exists( self._destination ):
				try:
					os.makedirs( self._destination )
				except Exception as e:
					ctxlogger.exception(
						TransferException,
						"Could not create: {}".format( str(e) ),
						orig_exc = e
					)

			with ctxlogger.context( 'file', name ):
				fullpath = os.path.join( self._destination, name )
				try:
					with open( fullpath, 'w' ) as outfile:
						outfile.write( data )
				except Exception as e:
					ctxlogger.exception(
						TransferException,
						"Could not write data: {}".format( str(e) ),
						orig_exc = e
					)
				try:
					os.chmod( fullpath, self._perms )
				except Exception as e:
					ctxlogger.exception(
						TransferException,
						"Could not set permissions: {}".format( str(e) ),
						orig_exc = e,
					)

	def get( self, pattern = '.*' ):
		"""
		pattern - a regexp to pick remote files (default '.*')
		"""
		data = {}

		with ctxlogger.context( 'address', self._source ):
			if not os.path.exists( self._source ):
				ctxlogger.exception(
					TransferException, 'Path does not exist'
				)

			if not os.path.isdir( self._source ):
				ctxlogger.exception(
					TransferException, 'Path is not a directory'
				)

			pattern = re.compile( pattern )
			for f in os.listdir( self._source ):
				with ctxlogger.context( 'file', f ):

					infile = os.path.join( self._source, f )
					if pattern.match( f ):
						with open( infile ) as indata:
							data[ f ] = indata.read()
					else:
						logger.debug( 'Ignoring' )
						continue

					if self._debug:
						continue

					if self._processed:
						if not os.path.exists( self._processed ):
							try:
								os.makedirs( self._processed )
							except Exception as e:
								ctxlogger.exception(
									TransferException,
									"Could not create {}: {}".format(
										self._processed,
										str(e)
									),
									orig_exc = e
								)

						dest = os.path.join( self._processed, f )
						logger.info(
							'Moving to {}'.format( self._processed )
						)

						try:
							os.rename( infile, dest )
						except Exception as e:
							ctxlogger.exception(
								TransferException,
								"Move failed: {}".format( str(e) ),
								orig_exc = e
							)
					else:
						logger.debug( 'Deleting' )
						try:
							os.unlink( infile )
						except Exception as e:
							ctxlogger.exception(
								TransferException,
								"Deletion failed: {}".format( str(e) ),
								orig_exc = e
							)

		return data

## EXPERIMENTAL
#class local_dispatcher( threading.Thread ):
#	"""
#	A threaded class that uses linux inotify support
#	to trigger events when local files are created.
#	"""

#	def __init__( self, **kwargs ):
#		super().__init__( **kwargs )
#		self.watcher = watcher.Watcher()
#		self.handlers = {}

#	def register( self, path, callobj ):
#		if not path in self.handlers:
#			self.handlers[ path ] = []
#		self.handlers[ path ].append( callobj )
#		self.watcher.add( path, inotify.IN_CLOSE_WRITE )

#	def run( self ):
#		while True:
#			for event in self.watcher.read(0):
#				if event.path in self.handlers:
#					for callobj in self.handlers[ event.path ]:
#						callobj()
#			time.sleep(1)



import unittest
from flask import Flask

from simpletransfers.http_server import http_server, wsgi_interface

post_xml = '<?xml version="1.0"?><test><element attrib="value" /></test>'
post_json = '{"test": {"element": {"@attrib": "value"}}}'
get_json = '[{ "serial": "0123456776543210", "pin":1234 }]'

class testHTTPServer( unittest.TestCase ):

	def setUp( self ):
		app = Flask( __name__ )
		app.config[ 'TESTING' ] = True

		app.add_url_rule(
			'/testpost', 'post', self._post, methods = [ 'POST' ]
		)
		app.add_url_rule( '/testget', 'get', self._get )
		app.add_url_rule(
			'/testget/serial/<serial>', 'get', self._get
		)

		self.app = app.test_client()

	@wsgi_interface
	def _post( self ):
		receiver = http_server()
		self.assertEquals(
			receiver.get()[ '_raw' ], post_xml
		)
		receiver.put( post_json, 'response.json' )

	@wsgi_interface
	def _get( self ):
		receiver = http_server()
		self.assertEquals(
			receiver.get()[ 'serial' ], '0123456776543210'
		)
		receiver.put( get_json, 'response.json' )

	def test_post( self ):
		response = self.app.post(
			'/testpost', data = post_xml, content_type = 'text/xml'
		)
		self.assertEquals( response.status_code, 200 )
		self.assertEquals(
			response.headers['Content-Type'], 
			'application/json'
		)
		self.assertEquals( response.get_data(), post_json )

	def test_get( self ):
		response = self.app.get( '/testget/serial/0123456776543210' )
		self.assertEquals( response.status_code, 200 )
		self.assertEquals(
			response.headers['Content-Type'], 
			'application/json'
		)
		self.assertEquals( response.get_data(), get_json )

		# test request via traditional query string too
		response = self.app.get( '/testget?serial=0123456776543210' )
		self.assertEquals( response.status_code, 200 )
		self.assertEquals(
			response.headers['Content-Type'], 
			'application/json'
		)
		self.assertEquals( response.get_data(), get_json )

	def test_incorrect_path(self):
		response = self.app.get( '/tests/doesnotexist' )
		self.assertEquals( response.status_code, 404 )

if __name__ == '__main__':
	unittest.main()



import json

from stubserver import StubServer
import unittest

from simpletransfers import http_client, TransferException

test_json = json.dumps( { 'quantity': 5 } )

class testHTTPClient( unittest.TestCase ):

	def setUp(self):
		self.server = StubServer(8095)
		self.server.run()

	def tearDown(self):
		self.server.stop()

	def test_get(self):
		self.server.expect(
			method = "GET",
			url = "/pin/serial/\d+$",
		).and_return(
			content = test_json,
			mime_type = "application/json",
		)

		obj = http_client( 'http://localhost:8095/pin/serial/658329293232' )
		contents = obj.get()

		result = json.loads( contents[ '658329293232' ] )
		self.assertEquals( 5, result[ 'quantity' ] )

	def test_post(self):
		capture = {}
		self.server.expect(
			method = "POST",
			url = "/serial",
			data_capture = capture,
		).and_return( reply_code = 200 )

		obj = http_client( 'http://localhost:8095/serial' )
		obj.put( json.dumps( { 'quantity': 5 } ) )

		result = json.loads( capture[ 'body' ] )
		self.assertEquals( 5, result[ 'quantity' ] )

	def test_with_500(self):
		capture = {}
		self.server.expect(
			method = "POST",
			url = "/allocate",
			data_capture = capture
		).and_return( reply_code = 500 )

		obj = http_client( 'http://localhost:8095/allocate' )
		self.assertRaises( TransferException, obj.put, test_json )

if __name__ == '__main__':
	unittest.main()



import os
import subprocess
import unittest

from simpletransfers.zip import zipfile

fullcontent = { 'one.txt': 'wibble\n', 'two.xml': 'bob\n' }
txtcontent = { 'one.txt': 'wibble\n' }

zipoutput = r'Archive:.*put\.zip\s+extracting: one\.txt\s+wibble\s+extracting: two\.xml\s+bob'

class testZipfile( unittest.TestCase ):

	def setUp( self ):
		self.local_dir = os.path.dirname( __file__ )

	def tearDown( self ):
		try:
			os.unlink( os.path.join( self.local_dir, 'put.zip' ) )
		except:
			pass

	def test_get( self ):
		with open( os.path.join( self.local_dir, 'test.zip' ) ) as zfh:
			zipcontent = zfh.read()

		zipobj = zipfile( zipcontent )

		content = zipobj.get()
		self.assertEquals( content, fullcontent )

		content = zipobj.get( pattern = r'.*\.txt' )
		self.assertEquals( content, txtcontent )

	def test_put( self ):
		zipobj = zipfile()
		zipcontent = zipobj.put( 'wibble\n', 'one.txt' )
		zipcontent = zipobj.put( 'bob\n', 'two.xml' )

		with open( os.path.join( self.local_dir, 'put.zip' ), 'w' ) as zfh:
			zfh.write( zipcontent )

		output = subprocess.check_output(
			[ 'unzip', '-c', os.path.join( self.local_dir, 'put.zip' ) ]
		)
		self.assertRegexpMatches( output, zipoutput )

if __name__ == '__main__':
	unittest.main()



from distutils.core import setup

setup(
	name = 'simpletransfers',
	version = '1.0.6',
	description = 'Simple file transfer library',
	author = 'Jesters Ghost',
	author_email = 'jestersghost@gmail.com',
	url = 'https://bitbucket.org/jestersghost/simpletransfers',
	requires = [ 'ctxlogger', 'Flask', 'paramiko', 'requests', 'suds' ],
	package_dir = { 'simpletransfers': '.' },
	packages = [
		'simpletransfers',
		'simpletransfers.tests',
	],
	package_data = { 'simpletransfers.tests': [ 'one.txt', 'test.zip', ] },
)
